@@ -30,6 +30,7 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/sets"
 
+	"knative.dev/networking/pkg/apis/networking"
 	fakekubeclient "knative.dev/pkg/client/injection/kube/client/fake"
 	fakeendpointsinformer "knative.dev/pkg/client/injection/kube/informers/core/v1/endpoints/fake"
 	fakeserviceinformer "knative.dev/pkg/client/injection/kube/informers/core/v1/service/fake"
@@ -38,7 +39,6 @@ import (
 	"knative.dev/pkg/ptr"
 	rtesting "knative.dev/pkg/reconciler/testing"
 	activatortest "knative.dev/serving/pkg/activator/testing"
-	"knative.dev/serving/pkg/apis/networking"
 	"knative.dev/serving/pkg/apis/serving"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
 	fakeservingclient "knative.dev/serving/pkg/client/injection/client/fake"
@@ -53,7 +53,7 @@ const (
 	testRevision  = "test-revision"
 
 	probeFreq     = 50 * time.Millisecond
-	updateTimeout = 6 * probeFreq
+	updateTimeout = 8 * probeFreq
 )
 
 // revisionCC1 - creates a revision with concurrency == 1.
@@ -385,7 +385,6 @@ func TestRevisionWatcher(t *testing.T) {
 			ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
 
 			updateCh := make(chan revisionDestsUpdate, len(tc.expectUpdates)+1)
-			defer close(updateCh)
 
 			// This gets closed up by revisionWatcher
 			destsCh := make(chan dests)
@@ -407,11 +406,12 @@ func TestRevisionWatcher(t *testing.T) {
 
 			waitInformers, err := controller.RunInformers(ctx.Done(), informer.Informer())
 			if err != nil {
-				t.Fatalf("Failed to start informers: %v", err)
+				t.Fatal("Failed to start informers:", err)
 			}
 			defer func() {
 				cancel()
 				waitInformers()
+				close(updateCh)
 			}()
 
 			rw := newRevisionWatcher(
@@ -441,7 +441,7 @@ func TestRevisionWatcher(t *testing.T) {
 				case update := <-updateCh:
 					updates = append(updates, update)
 				case <-time.After(updateTimeout):
-					t.Error("Timed out waiting for update event")
+					t.Fatal("Timed out waiting for update event")
 				}
 			}
 			if got, want := rw.podsAddressable, !tc.noPodAddressability; got != want {
@@ -469,7 +469,7 @@ func TestRevisionWatcher(t *testing.T) {
 func assertChClosed(t *testing.T, ch chan struct{}) {
 	defer func() {
 		if r := recover(); r == nil {
-			t.Error("the channel was not closed")
+			t.Error("The channel was not closed")
 		}
 	}()
 	select {
@@ -480,7 +480,7 @@ func assertChClosed(t *testing.T, ch chan struct{}) {
 	}
 }
 
-func epSubset(port int32, portName string, ips []string, notReadyIps []string) *corev1.EndpointSubset {
+func epSubset(port int32, portName string, ips, notReadyIps []string) *corev1.EndpointSubset {
 	ss := &corev1.EndpointSubset{
 		Ports: []corev1.EndpointPort{{
 			Name: portName,
@@ -500,7 +500,7 @@ func ep(revL string, port int32, portName string, ips ...string) *corev1.Endpoin
 	return epNotReady(revL, port, portName, ips, nil)
 }
 
-func epNotReady(revL string, port int32, portName string, readyIps []string, notReadyIps []string) *corev1.Endpoints {
+func epNotReady(revL string, port int32, portName string, readyIps, notReadyIps []string) *corev1.Endpoints {
 	return &corev1.Endpoints{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: revL + "-ep",
@@ -707,15 +707,16 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 
 			waitInformers, err := controller.RunInformers(ctx.Done(), endpointsInformer.Informer())
 			if err != nil {
-				t.Fatalf("Failed to start informers: %v", err)
+				t.Fatal("Failed to start informers:", err)
 			}
+
+			rbm := newRevisionBackendsManagerWithProbeFrequency(ctx, rt, probeFreq)
 			defer func() {
 				cancel()
 				waitInformers()
+				waitForRevisionBackedMananger(t, rbm)
 			}()
 
-			rbm := newRevisionBackendsManagerWithProbeFrequency(ctx, rt, probeFreq)
-
 			for _, ep := range tc.endpointsArr {
 				fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ep)
 				endpointsInformer.Informer().GetIndexer().Add(ep)
@@ -741,13 +742,158 @@ func TestRevisionBackendManagerAddEndpoint(t *testing.T) {
 			if got, want := revDests, tc.expectDests; !cmp.Equal(got, want) {
 				t.Errorf("RevisionDests = %v, want: %v, diff(-want,+got):%s\n", got, want, cmp.Diff(want, got))
 			}
-
-			cancel()
-			waitForRevisionBackedMananger(t, rbm)
 		})
 	}
 }
 
+func emptyDests() dests {
+	return dests{
+		ready:    sets.NewString(),
+		notReady: sets.NewString(),
+	}
+}
+
+func TestCheckDestsReadyToNotReady(t *testing.T) {
+	// This test verifies the edge behaviour when a pod
+	// previously in the ready sed moved to non ready set
+	// and now must be re-probed.
+	ctx, cancel, _ := rtesting.SetupFakeContextWithCancel(t)
+
+	svc := privateSKSService(
+		types.NamespacedName{Namespace: testNamespace, Name: testRevision},
+		"129.0.0.1",
+		[]corev1.ServicePort{{Name: "http", Port: 1234}},
+	)
+	fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
+	si := fakeserviceinformer.Get(ctx)
+	si.Informer().GetIndexer().Add(svc)
+
+	waitInformers, err := controller.RunInformers(ctx.Done(), si.Informer())
+	if err != nil {
+		t.Fatal("Failed to start informers:", err)
+	}
+	defer func() {
+		cancel()
+		waitInformers()
+	}()
+
+	fakeRT := activatortest.FakeRoundTripper{
+		ExpectHost: testRevision,
+		ProbeHostResponses: map[string][]activatortest.FakeResponse{
+			"10.10.1.1": {{
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				// Finally!
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+			"10.10.1.2": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+			"10.10.1.3": {{
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}, {
+				Err: errors.New("clusterIP transport error"),
+			}, {
+				Code: http.StatusOK,
+				Body: queue.Name,
+			}},
+		},
+	}
+
+	// Make it buffered,so that we can make the test linear.
+	uCh := make(chan revisionDestsUpdate, 1)
+	dCh := make(chan struct{})
+	rw := &revisionWatcher{
+		clusterIPHealthy: true,
+		podsAddressable:  true,
+		rev:              types.NamespacedName{Namespace: testNamespace, Name: testRevision},
+		updateCh:         uCh,
+		serviceLister:    si.Lister(),
+		logger:           TestLogger(t),
+		stopCh:           dCh,
+		transport:        network.RoundTripperFunc(fakeRT.RT),
+	}
+	// Initial state. Both are ready.
+	cur := dests{
+		ready:    sets.NewString("10.10.1.3", "10.10.1.2"),
+		notReady: sets.NewString("10.10.1.1"),
+	}
+	rw.checkDests(cur, emptyDests())
+	select {
+	case u := <-uCh:
+		if got, want := len(u.Dests), 2; got != want {
+			t.Fatalf("NumReady = %d, want: %d, update.Dests = %v", got, want, u.Dests)
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+	// Repeating should be a no-op.
+	rw.checkDests(cur, emptyDests())
+	select {
+	case u := <-uCh:
+		t.Fatal("Unexpected update", u)
+	default:
+		// Success.
+	}
+
+	// Now swing the ready to the non ready, and it should fail the probe.
+	prev := cur
+
+	cur = dests{
+		ready:    sets.NewString("10.10.1.2"),
+		notReady: sets.NewString("10.10.1.1", "10.10.1.3"),
+	}
+	rw.checkDests(cur, prev)
+	select {
+	case u := <-uCh:
+		// The 10.10.1.3 should be not ready now.
+		if got, want := len(u.Dests), 1; got != want {
+			t.Fatalf("NumReady = %d, want: %d, update.Dests = %v", got, want, u.Dests)
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+
+	// Now just re-probe the same and it should succeed (see the RT setup above).
+	rw.checkDests(cur, prev)
+	select {
+	case u := <-uCh:
+		if got, want := len(u.Dests), 2; got != want {
+			t.Fatalf("NumReady = %d, want: %d, update.Dests = %v", got, want, u.Dests)
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+
+	// Timer update.
+	prev.ready.Delete("10.10.1.3")
+	// The last one should finally succeed.
+	rw.checkDests(cur, prev)
+	select {
+	case u := <-uCh:
+		if got, want := len(u.Dests), 3; got != want {
+			t.Fatalf("NumReady = %d, want: %d, update.Dests = %v", got, want, u.Dests)
+		}
+	default:
+		t.Error("Expected update but it never went out.")
+	}
+}
+
 func TestCheckDests(t *testing.T) {
 	// This test covers some edge cases in `checkDests` which are next to impossible to
 	// test via tests above.
@@ -765,7 +911,7 @@ func TestCheckDests(t *testing.T) {
 
 	waitInformers, err := controller.RunInformers(ctx.Done(), si.Informer())
 	if err != nil {
-		t.Fatalf("Failed to start informers: %v", err)
+		t.Fatal("Failed to start informers:", err)
 	}
 	defer func() {
 		cancel()
@@ -784,7 +930,10 @@ func TestCheckDests(t *testing.T) {
 		logger:           TestLogger(t),
 		stopCh:           dCh,
 	}
-	rw.checkDests(dests{ready: sets.NewString("10.1.1.5"), notReady: sets.NewString("10.1.1.6")})
+	rw.checkDests(dests{
+		ready:    sets.NewString("10.1.1.5"),
+		notReady: sets.NewString("10.1.1.6"),
+	}, emptyDests())
 	select {
 	case <-uCh:
 		// Success.
@@ -793,7 +942,10 @@ func TestCheckDests(t *testing.T) {
 	}
 
 	close(dCh)
-	rw.checkDests(dests{ready: sets.NewString("10.1.1.5"), notReady: sets.NewString("10.1.1.6")})
+	rw.checkDests(dests{
+		ready:    sets.NewString("10.1.1.5"),
+		notReady: sets.NewString("10.1.1.6"),
+	}, emptyDests())
 	select {
 	case <-uCh:
 		t.Error("Expected no update but got one")
@@ -819,7 +971,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 
 	waitInformers, err := controller.RunInformers(ctx.Done(), si.Informer())
 	if err != nil {
-		t.Fatalf("Failed to start informers: %v", err)
+		t.Fatal("Failed to start informers:", err)
 	}
 	defer func() {
 		cancel()
@@ -878,8 +1030,9 @@ func TestCheckDestsSwinging(t *testing.T) {
 		podsAddressable: true,
 		transport:       network.RoundTripperFunc(fakeRT.RT),
 	}
+
 	// First not ready, second good, clusterIP: not ready.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")}, emptyDests())
 	want := revisionDestsUpdate{
 		Rev:           types.NamespacedName{Namespace: testNamespace, Name: testRevision},
 		ClusterIPDest: "",
@@ -896,7 +1049,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Second gone, first becomes ready, clusterIP still not ready.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.1:1234")
@@ -908,7 +1061,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Second is back, but not healthy yet.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		// No update should be sent out, since there's only healthy pod, same as above.
@@ -917,7 +1070,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// All pods are happy now.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.2:1234", "10.0.0.1:1234")
@@ -929,7 +1082,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Make sure we do not send out redundant updates.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		t.Errorf("Expected no update, but got %#v", got)
@@ -938,7 +1091,10 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Add a notReady pod, but it's not ready. No update should be sent.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"), notReady: sets.NewString("10.0.0.4:1234")})
+	rw.checkDests(dests{
+		ready:    sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"),
+		notReady: sets.NewString("10.0.0.4:1234"),
+	}, emptyDests())
 	select {
 	case got := <-uCh:
 		t.Errorf("Expected no update, but got %#v", got)
@@ -947,7 +1103,10 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// The notReady pod is now ready!
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"), notReady: sets.NewString("10.0.0.4:1234")})
+	rw.checkDests(dests{
+		ready:    sets.NewString("10.0.0.1:1234", "10.0.0.2:1234"),
+		notReady: sets.NewString("10.0.0.4:1234"),
+	}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.1:1234", "10.0.0.2:1234", "10.0.0.4:1234")
@@ -959,7 +1118,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Swing to a different pods.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.3:1234", "10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.3:1234", "10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.2:1234", "10.0.0.3:1234")
@@ -971,7 +1130,7 @@ func TestCheckDestsSwinging(t *testing.T) {
 	}
 
 	// Scale down by 1.
-	rw.checkDests(dests{ready: sets.NewString("10.0.0.2:1234")})
+	rw.checkDests(dests{ready: sets.NewString("10.0.0.2:1234")}, emptyDests())
 	select {
 	case got := <-uCh:
 		want.Dests = sets.NewString("10.0.0.2:1234")
@@ -1000,12 +1159,8 @@ func TestRevisionDeleted(t *testing.T) {
 	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(ep)
 	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
 	if err != nil {
-		t.Fatalf("Failed to start informers: %v", err)
+		t.Fatal("Failed to start informers:", err)
 	}
-	defer func() {
-		cancel()
-		waitInformers()
-	}()
 
 	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1)
 	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
@@ -1013,14 +1168,18 @@ func TestRevisionDeleted(t *testing.T) {
 	ri.Informer().GetIndexer().Add(rev)
 
 	fakeRT := activatortest.FakeRoundTripper{}
-	rt := network.RoundTripperFunc(fakeRT.RT)
+	rbm := newRevisionBackendsManagerWithProbeFrequency(ctx, network.RoundTripperFunc(fakeRT.RT), probeFreq)
+	defer func() {
+		cancel()
+		waitInformers()
+		waitForRevisionBackedMananger(t, rbm)
+	}()
 
-	rbm := newRevisionBackendsManager(ctx, rt)
 	// Make some movements.
 	ei.Informer().GetIndexer().Add(ep)
 	select {
 	case <-rbm.updates():
-	case <-time.After(time.Second * 2):
+	case <-time.After(updateTimeout):
 		t.Error("Timedout waiting for initial response")
 	}
 	// Now delete the endpoints.
@@ -1031,9 +1190,6 @@ func TestRevisionDeleted(t *testing.T) {
 	case <-time.After(updateTimeout):
 		// Wait to make sure the callbacks are executed.
 	}
-
-	cancel()
-	waitForRevisionBackedMananger(t, rbm)
 }
 
 func TestServiceDoesNotExist(t *testing.T) {
@@ -1045,12 +1201,8 @@ func TestServiceDoesNotExist(t *testing.T) {
 	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(eps)
 	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
 	if err != nil {
-		t.Fatalf("Failed to start informers: %v", err)
+		t.Fatal("Failed to start informers:", err)
 	}
-	defer func() {
-		cancel()
-		waitInformers()
-	}()
 
 	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1)
 	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
@@ -1072,9 +1224,13 @@ func TestServiceDoesNotExist(t *testing.T) {
 			}},
 		},
 	}
-	rt := network.RoundTripperFunc(fakeRT.RT)
+	rbm := newRevisionBackendsManagerWithProbeFrequency(ctx, network.RoundTripperFunc(fakeRT.RT), probeFreq)
+	defer func() {
+		cancel()
+		waitInformers()
+		waitForRevisionBackedMananger(t, rbm)
+	}()
 
-	rbm := newRevisionBackendsManager(ctx, rt)
 	// Make some movements to generate a checkDests call.
 	ei.Informer().GetIndexer().Add(eps)
 	select {
@@ -1085,9 +1241,6 @@ func TestServiceDoesNotExist(t *testing.T) {
 		t.Errorf("Unexpected update, should have had none: %v", x)
 	case <-time.After(updateTimeout):
 	}
-
-	cancel()
-	waitForRevisionBackedMananger(t, rbm)
 }
 
 func TestServiceMoreThanOne(t *testing.T) {
@@ -1099,13 +1252,8 @@ func TestServiceMoreThanOne(t *testing.T) {
 	fakekubeclient.Get(ctx).CoreV1().Endpoints(testNamespace).Create(eps)
 	waitInformers, err := controller.RunInformers(ctx.Done(), ei.Informer())
 	if err != nil {
-		t.Fatalf("Failed to start informers: %v", err)
+		t.Fatal("Failed to start informers:", err)
 	}
-	defer func() {
-		cancel()
-		waitInformers()
-	}()
-
 	rev := revisionCC1(types.NamespacedName{Namespace: testNamespace, Name: testRevision}, networking.ProtocolHTTP1)
 	fakeservingclient.Get(ctx).ServingV1().Revisions(testNamespace).Create(rev)
 	ri := fakerevisioninformer.Get(ctx)
@@ -1119,7 +1267,7 @@ func TestServiceMoreThanOne(t *testing.T) {
 			[]corev1.ServicePort{{Name: "http", Port: 1234}},
 		)
 		// Modify the name so both can be created.
-		svc.Name = svc.Name + num
+		svc.Name += num
 		fakekubeclient.Get(ctx).CoreV1().Services(testNamespace).Create(svc)
 		si := fakeserviceinformer.Get(ctx)
 		si.Informer().GetIndexer().Add(svc)
@@ -1140,19 +1288,20 @@ func TestServiceMoreThanOne(t *testing.T) {
 			}},
 		},
 	}
+	rbm := newRevisionBackendsManagerWithProbeFrequency(ctx, network.RoundTripperFunc(fakeRT.RT), probeFreq)
+	defer func() {
+		cancel()
+		waitInformers()
+		waitForRevisionBackedMananger(t, rbm)
+	}()
 
-	rt := network.RoundTripperFunc(fakeRT.RT)
-	rbm := newRevisionBackendsManager(ctx, rt)
 	ei.Informer().GetIndexer().Add(eps)
 	select {
 	case x := <-rbm.updates():
 		// We can't probe endpoints (see RT above) and we can't get to probe
 		// cluster IP. But if the service is accessible then we will and probing will
 		// succeed since RT has no rules for that.
-		t.Errorf("Unexpected update, should have had none: %v", x)
+		t.Errorf("Unexpected update, should have had none: %#v", x)
 	case <-time.After(updateTimeout):
 	}
-
-	cancel()
-	waitForRevisionBackedMananger(t, rbm)
 }
