@@ -17,6 +17,8 @@ package traffic
 
 import (
 	"context"
+	"errors"
+	"log"
 	"os"
 	"testing"
 	"time"
@@ -24,8 +26,10 @@ import (
 	"github.com/google/go-cmp/cmp"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime"
 
+	network "knative.dev/networking/pkg"
 	net "knative.dev/networking/pkg/apis/networking"
 	"knative.dev/pkg/ptr"
 	"knative.dev/serving/pkg/apis/serving"
@@ -34,7 +38,6 @@ import (
 	informers "knative.dev/serving/pkg/client/informers/externalversions"
 	listers "knative.dev/serving/pkg/client/listers/serving/v1"
 	"knative.dev/serving/pkg/gc"
-	"knative.dev/serving/pkg/network"
 	"knative.dev/serving/pkg/reconciler/route/config"
 	"knative.dev/serving/pkg/reconciler/route/domains"
 	. "knative.dev/serving/pkg/testing/v1"
@@ -80,6 +83,9 @@ var (
 	configLister listers.ConfigurationLister
 	revLister    listers.RevisionLister
 
+	// fake lister to return API error.
+	revErrorLister listers.RevisionLister
+
 	cmpOpts = []cmp.Option{cmp.AllowUnexported(Config{})}
 )
 
@@ -99,6 +105,8 @@ func setUp() {
 	revInformer := servingInformer.Serving().V1().Revisions()
 	revLister = revInformer.Lister()
 
+	revErrorLister = revFakeErrorLister{}
+
 	// Add these test objects to the informers.
 	objs := []runtime.Object{
 		unreadyConfig, unreadyRev,
@@ -167,10 +175,11 @@ func TestBuildTrafficConfigurationVanilla(t *testing.T) {
 }
 
 func testRouteWithTrafficTargets(trafficTarget RouteOption) *v1.Route {
-	return Route(testNamespace, "test-route", WithRouteLabel(map[string]string{"route": "test-route"}), trafficTarget)
+	return Route(testNamespace, "test-route",
+		WithRouteLabel(map[string]string{"route": "test-route"}), trafficTarget)
 }
 
-func TestBuildTrafficConfiguration_NoNameRevision(t *testing.T) {
+func TestBuildTrafficConfigurationNoNameRevision(t *testing.T) {
 	tts := v1.TrafficTarget{
 		RevisionName: goodNewRev.Name,
 		Percent:      ptr.Int64(100),
@@ -209,7 +218,7 @@ func TestBuildTrafficConfiguration_NoNameRevision(t *testing.T) {
 }
 
 // The vanilla use case of 100% directing to latest revision of an inactive configuration.
-func TestBuildTrafficConfiguration_VanillaScaledToZero(t *testing.T) {
+func TestBuildTrafficConfigurationVanillaScaledToZero(t *testing.T) {
 	tts := v1.TrafficTarget{
 		ConfigurationName: inactiveConfig.Name,
 		Percent:           ptr.Int64(100),
@@ -252,7 +261,7 @@ func TestBuildTrafficConfiguration_VanillaScaledToZero(t *testing.T) {
 }
 
 // Transitioning from one good config to another by splitting traffic.
-func TestBuildTrafficConfiguration_TwoConfigs(t *testing.T) {
+func TestBuildTrafficConfigurationTwoConfigs(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{
 			DefaultTarget: {{
@@ -317,7 +326,7 @@ func TestBuildTrafficConfiguration_TwoConfigs(t *testing.T) {
 }
 
 // Splitting traffic between a fixed revision and the latest revision (canary).
-func TestBuildTrafficConfiguration_Canary(t *testing.T) {
+func TestBuildTrafficConfigurationCanary(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{
 			DefaultTarget: {{
@@ -381,7 +390,7 @@ func TestBuildTrafficConfiguration_Canary(t *testing.T) {
 }
 
 // Splitting traffic between latest revision and a fixed revision which is also latest.
-func TestBuildTrafficConfiguration_Consolidated(t *testing.T) {
+func TestBuildTrafficConfigurationConsolidated(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{
 			DefaultTarget: {{
@@ -498,7 +507,7 @@ func TestBuildTrafficConfiguration_Consolidated(t *testing.T) {
 }
 
 // Splitting traffic between a two fixed revisions.
-func TestBuildTrafficConfiguration_TwoFixedRevisions(t *testing.T) {
+func TestBuildTrafficConfigurationTwoFixedRevisions(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{
 			DefaultTarget: {{
@@ -562,7 +571,7 @@ func TestBuildTrafficConfiguration_TwoFixedRevisions(t *testing.T) {
 }
 
 // Splitting traffic between a two fixed revisions of two configurations.
-func TestBuildTrafficConfiguration_TwoFixedRevisionsFromTwoConfigurations(t *testing.T) {
+func TestBuildTrafficConfigurationTwoFixedRevisionsFromTwoConfigurations(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{
 			DefaultTarget: {{
@@ -627,7 +636,7 @@ func TestBuildTrafficConfiguration_TwoFixedRevisionsFromTwoConfigurations(t *tes
 }
 
 // One fixed, two named targets for newer stuffs.
-func TestBuildTrafficConfiguration_Preliminary(t *testing.T) {
+func TestBuildTrafficConfigurationPreliminary(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{
 			DefaultTarget: {{
@@ -719,23 +728,24 @@ func TestBuildTrafficConfiguration_Preliminary(t *testing.T) {
 			niceNewRev.Name: niceNewRev,
 		},
 	}
-	if tc, err := BuildTrafficConfiguration(configLister, revLister, testRouteWithTrafficTargets(WithSpecTraffic(v1.TrafficTarget{
-		RevisionName: goodOldRev.Name,
-		Percent:      ptr.Int64(100),
-	}, v1.TrafficTarget{
-		Tag:          "beta",
-		RevisionName: goodNewRev.Name,
-	}, v1.TrafficTarget{
-		Tag:               "alpha",
-		ConfigurationName: niceConfig.Name,
-	}))); err != nil {
-		t.Errorf("Unexpected error %v", err)
-	} else if want, got := tc, expected; !cmp.Equal(want, got, cmpOpts...) {
-		t.Errorf("Unexpected traffic diff (-want +got): %v", cmp.Diff(want, got, cmpOpts...))
+	if tc, err := BuildTrafficConfiguration(configLister, revLister,
+		testRouteWithTrafficTargets(WithSpecTraffic(v1.TrafficTarget{
+			RevisionName: goodOldRev.Name,
+			Percent:      ptr.Int64(100),
+		}, v1.TrafficTarget{
+			Tag:          "beta",
+			RevisionName: goodNewRev.Name,
+		}, v1.TrafficTarget{
+			Tag:               "alpha",
+			ConfigurationName: niceConfig.Name,
+		}))); err != nil {
+		t.Error("Unexpected error:", err)
+	} else if got, want := tc, expected; !cmp.Equal(want, got, cmpOpts...) {
+		t.Errorf("Unexpected traffic diff (-want +got):\n%s", cmp.Diff(want, got, cmpOpts...))
 	}
 }
 
-func TestBuildTrafficConfiguration_MissingConfig(t *testing.T) {
+func TestBuildTrafficConfigurationMissingConfig(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{},
 		Configurations: map[string]*v1.Configuration{
@@ -771,7 +781,7 @@ func TestBuildTrafficConfiguration_MissingConfig(t *testing.T) {
 	}
 }
 
-func TestBuildTrafficConfiguration_NotRoutableRevision(t *testing.T) {
+func TestBuildTrafficConfigurationNotRoutableRevision(t *testing.T) {
 	expected := &Config{
 		Targets:        map[string]RevisionTargets{},
 		Configurations: map[string]*v1.Configuration{},
@@ -788,7 +798,7 @@ func TestBuildTrafficConfiguration_NotRoutableRevision(t *testing.T) {
 	}
 }
 
-func TestBuildTrafficConfiguration_NotRoutableConfiguration(t *testing.T) {
+func TestBuildTrafficConfigurationNotRoutableConfiguration(t *testing.T) {
 	expected := &Config{
 		Targets:        map[string]RevisionTargets{},
 		Configurations: map[string]*v1.Configuration{unreadyConfig.Name: unreadyConfig},
@@ -805,7 +815,7 @@ func TestBuildTrafficConfiguration_NotRoutableConfiguration(t *testing.T) {
 	}
 }
 
-func TestBuildTrafficConfiguration_EmptyConfiguration(t *testing.T) {
+func TestBuildTrafficConfigurationEmptyConfiguration(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{},
 		Configurations: map[string]*v1.Configuration{
@@ -825,7 +835,7 @@ func TestBuildTrafficConfiguration_EmptyConfiguration(t *testing.T) {
 	}
 }
 
-func TestBuildTrafficConfiguration_EmptyAndFailedConfigurations(t *testing.T) {
+func TestBuildTrafficConfigurationEmptyAndFailedConfigurations(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{},
 		Configurations: map[string]*v1.Configuration{
@@ -848,7 +858,7 @@ func TestBuildTrafficConfiguration_EmptyAndFailedConfigurations(t *testing.T) {
 	}
 }
 
-func TestBuildTrafficConfiguration_FailedAndEmptyConfigurations(t *testing.T) {
+func TestBuildTrafficConfigurationFailedAndEmptyConfigurations(t *testing.T) {
 	expected := &Config{
 		Targets: map[string]RevisionTargets{},
 		Configurations: map[string]*v1.Configuration{
@@ -871,7 +881,7 @@ func TestBuildTrafficConfiguration_FailedAndEmptyConfigurations(t *testing.T) {
 	}
 }
 
-func TestBuildTrafficConfiguration_MissingRevision(t *testing.T) {
+func TestBuildTrafficConfigurationMissingRevision(t *testing.T) {
 	expected := &Config{
 		Targets:        map[string]RevisionTargets{},
 		Configurations: map[string]*v1.Configuration{goodConfig.Name: goodConfig},
@@ -897,6 +907,34 @@ func TestBuildTrafficConfiguration_MissingRevision(t *testing.T) {
 	}
 }
 
+var errAPI = errors.New("failed to connect API")
+
+type revFakeErrorLister struct{}
+
+func (l revFakeErrorLister) Get(name string) (*v1.Revision, error) {
+	return nil, errAPI
+}
+
+func (l revFakeErrorLister) List(selector labels.Selector) ([]*v1.Revision, error) {
+	log.Panic("not implemented")
+	return nil, nil
+}
+
+func (l revFakeErrorLister) Revisions(namespace string) listers.RevisionNamespaceLister {
+	return l
+}
+
+func TestBuildTrafficConfigurationFailedGetRevision(t *testing.T) {
+	_, err := BuildTrafficConfiguration(configLister, revErrorLister, testRouteWithTrafficTargets(WithSpecTraffic(v1.TrafficTarget{
+		RevisionName: goodNewRev.Name,
+		Percent:      ptr.Int64(50)})))
+	if err != nil && err.Error() != errAPI.Error() {
+		t.Errorf("err: %s, want: %s", err.Error(), errAPI.Error())
+	} else if err == nil {
+		t.Errorf("err: %s, want: no error", errAPI.Error())
+	}
+}
+
 func TestRoundTripping(t *testing.T) {
 	expected := []v1.TrafficTarget{{
 		RevisionName:   goodOldRev.Name,
@@ -928,10 +966,10 @@ func TestRoundTripping(t *testing.T) {
 	} else {
 		targets, err := tc.GetRevisionTrafficTargets(getContext(), route)
 		if err != nil {
-			t.Errorf("Unexpected error %v", err)
+			t.Error("Unexpected error:", err)
 		}
-		if want, got := expected, targets; !cmp.Equal(want, got) {
-			t.Errorf("Unexpected traffic diff (-want +got): %v", cmp.Diff(want, got))
+		if got, want := targets, expected; !cmp.Equal(want, got) {
+			t.Errorf("Unexpected traffic diff (-want +got):\n%s", cmp.Diff(want, got))
 		}
 	}
 }
